@@ -15,21 +15,35 @@ class ImageDataPoint(
     MarkAsCompletedMixin,
     MarkAsReviewMixin,
 ):
-    def setup(self):
-        # Call mixins' setup methods
-        self.setup_img()
+    is_modified = False
+    is_loaded = False
+    needs_refresh = False
+
+    def load(self):
+        self.load_img()
         self.setup_mark_selected()
         self.setup_mark_deleted()
         self.setup_mark_completed()
         self.setup_mark_review()
 
+    def unload(self):
+        self.unload_img_bytes()
+
+    def setup(self):
+        # Do nothing here for speedy instantiantion
+        # Instead, call `.load()` when needed
+        pass
+
     def setup_view(self):
-        self.view = VBox([self.img])
-        self.view.layout = Layout(width=f"{self.width}%")
-        self.view.add_class(f"{CSS_NAMES.IMG_BOX_CONTAINER}")
+        # Instantiate to blank view for speedy instantiantion
+        self.view = None
 
     def update_view(self):
-        self.view.children = [self.img]
+        if self.is_loaded:
+            self.view = VBox([])
+            self.view.layout = Layout(width=f"{self.width}%")
+            self.view.add_class(f"{CSS_NAMES.IMG_BOX_CONTAINER}")
+            self.view.children = [self.img]
 
     def setup_mouse_interaction(
         self,
@@ -68,6 +82,14 @@ class ImageDataPoint(
         )
         ev.on_dom_event(mouse_interaction)
 
+    def __repr__(self):
+        if self.view is None:
+            if not self.is_loaded:
+                return f"Unloaded {self.__class__}. Call `.load()` and `.update_view()`"
+            else:
+                return f"Call `.update_view()` to view image"
+        return ""
+
 
 class ImageWithLabels(ImageDataPoint, ClassificationLabelsMixin, NoteMixin):
     ""
@@ -96,13 +118,16 @@ class ImageWithLabels(ImageDataPoint, ClassificationLabelsMixin, NoteMixin):
 
         super().__init__(source=source, width=width)
 
-    def setup(self):
-        # Call mixins' setup methods
-        super().setup()
+    def load(self):
+        super().load()
         self.setup_note()
         self.setup_mouse_interaction()
         self.setup_labelling()
 
+    def setup(self):
+        # Call mixins' setup methods
+        super().setup()
+
     def sync_dset_width_slider(self):
         "Syncs `self.view`'s width to the dataset's width slider"
 
@@ -123,20 +148,45 @@ class ImageWithLabels(ImageDataPoint, ClassificationLabelsMixin, NoteMixin):
         Sets up mouse click events to update elements of the parent dataset
         when the image is clicked
         """
-        self.sync_dset_width_slider()
         if self.dset is not None:
             self.setup_mouse_interaction(
                 on_mouse_click_callbacks=[
-                    self.dset.update_info,
                     self.dset.update_batch_labelling_descriptions,
+                    self.dset.update_info,
                 ]
             )
 
+    def setup_view(self):
+        self.view = None
+
+    def update_view_minimal(self):
+        # Not needed? Better safe than sorry.
+        if self.is_loaded:
+            self.view.children = [self.img]
+            self.view.remove_class(CSS_NAMES.IMG_BOX_CONTAINER)
+            self.view.add_class(CSS_NAMES.IMG_BOX_CONTAINER_MINIMAL)
+
     def update_view(self):
-        self.setup_dset_interaction_events()
-        self.view.children = [
-            self.img,
-            self.note,
-            HBox([self.toggle_note_button, self.searchbox]),
-            self.label_buttons.buttons,
-        ]
+        if self.is_loaded:
+            if self.view is None:
+                # Don't reinitialise if already created
+                self.view = VBox([])
+
+                if self.dset is not None:
+                    self.view.layout = Layout(width=f"{self.dset.width_slider.value}%")
+                else:
+                    self.view.layout = Layout(width=f"{self.width}%")
+
+                self.sync_dset_width_slider()
+                self.setup_dset_interaction_events()
+
+            self.view.children = [
+                self.img,
+                self.note,
+                HBox([self.toggle_note_button, self.searchbox]),
+                self.label_buttons.buttons,
+            ]
+            self.view.add_class(f"{CSS_NAMES.IMG_BOX_CONTAINER}")
+            self.view.remove_class(CSS_NAMES.IMG_BOX_CONTAINER_MINIMAL)
+        else:
+            pass
