@@ -4,16 +4,24 @@ from datawidgets.data.image import *
 
 
 class ImageLoaderMixin:
-    def setup_img(self):
-        # Create image widget
-        self.img = widgets.Image(value=self.load_img_bytes(), width=f"100%")
-        self.img.add_class(CSS_NAMES.IMG_BOX)
+    ""
+
+    def unload_img_bytes(self):
+        self.img.value = b""
+        self.needs_refresh = True
 
     def load_img_bytes(self):
+        self.needs_refresh = False
         if "https://" in self.source:
-            return requests.get(self.source).content
+            self.img.value = requests.get(self.source).content
         else:
-            return Path(self.source).read_bytes()
+            self.img.value = Path(self.source).read_bytes()
+
+    def load_img(self):
+        self.is_loaded = True
+        self.img = widgets.Image(width=f"100%")
+        self.img.add_class(CSS_NAMES.IMG_BOX)
+        self.load_img_bytes()
 
 
 class NoteMixin:
@@ -64,7 +72,6 @@ class ClassificationLabelsMixin:
     _works_with = "ImageWithLabels"
 
     def setup_labelling(self):
-        self.is_modified = False
         self.label_buttons = ClassificationLabelButtons(
             class_map=self.class_map,
             labels=self._labels,
