@@ -40,20 +40,20 @@ class BatchClassificationLabelsMixin:
         # it but it's here for safety as setup functions don't have an order
         if hasattr(self, "class_map_negative_buttons"):
             self.batch_remove_button.items = flatten(
-                [i.labels for i in self.selected_items]
+                [item.labels for item in self.selected_items]
             )
 
     def add_batch_label(self, change):
         if self.batch_add_value in self.classes:
-            for i in self.selected_items:
-                if not i.max_labels_selected:
-                    i.add_label(self.batch_add_value)
+            for item in self.selected_items:
+                if not item.max_labels_selected:
+                    item.add_label(self.batch_add_value)
             self.batch_add_button.v_model = ""
 
     def remove_batch_label(self, change):
         if self.batch_remove_value in self.classes:
-            for i in self.selected_items:
-                i.remove_label(self.batch_remove_value)
+            for item in self.selected_items:
+                item.remove_label(self.batch_remove_value)
             self.batch_remove_button.v_model = ""
 
 
@@ -74,18 +74,18 @@ class WidthSliderMixin:
 
 class SelectionMixin:
     ""
+    selection_source = "active_datapoints"
 
     def unselect_all(self, *args, callbacks=[]):
-        for item in self.active_datapoints:
-            item["item"].unselect()
+        for item in getattr(self, self.selection_source):
+            item.unselect()
 
     def select_all(self, *args, callbacks=[]):
-        for item in self.active_datapoints:
-            item["item"].select()
+        for item in getattr(self, self.selection_source):
+            item.select()
 
     def invert_selection(self, *args, callbacks=[]):
-        for i in self.active_datapoints:
-            item = i["item"]
+        for item in getattr(self, self.selection_source):
             if item.is_selected:
                 item.unselect()
             else:
@@ -122,14 +122,12 @@ class SelectionMixin:
         return button
 
 
-class CinemaNetSimilarityMixin:
+class EmbeddingSimilarityMixin:
     from scipy import spatial
 
     # TODO: Unused
     def setup_similarity(self, callbacks=[]):
-        menu = widgets.Dropdown(
-            options=["cinemanet_embedding", "cinemanet_probabilities"]
-        )
+        menu = widgets.Dropdown(options=self.embedding_colname)
         self.compare_key_options = menu
 
     def disable_similarity_button(self, *args):
@@ -154,7 +152,7 @@ class CinemaNetSimilarityMixin:
             self.log(f"Computed similarity")
             self.log(f"Similarity indices: sorting_indices")
 
-            self.datapoints[sorting_indices[0]]["item"].unselect()
+            self.datapoints[sorting_indices[0]].unselect()
             self.filter_and_mutate_dataset(sorting_indices)
             self.refresh()
             # self.reset(df=df.loc[similarity.sort_values().index.values])
@@ -164,6 +162,7 @@ class CinemaNetSimilarityMixin:
                 cb()
 
         button = widgets.Button(description="Sort By Similarity")
+        button.layout.width = "200px"
         button.on_click(calc_similarity)
         return button
 
@@ -171,18 +170,33 @@ class CinemaNetSimilarityMixin:
         self,
         source: Union[pd.Series, pd.DataFrame],
         target: pd.Series,
-        # compare_key: str = "cinemanet_embedding",
-        metric_func: Callable = spatial.distance.cosine,
     ):
         source = convert_single_row_to_series(source)
 
         compare_key = self.compare_key_options.value
         self.log(f"Computing similarity by {compare_key}")
 
-        if compare_key in ["cinemanet_embedding", "cinemanet_probabilities"]:
-            return metric_func(source[compare_key], target[compare_key])
-        else:
-            raise RuntimeError(f"Invalid `compare_key`: {compare_key}")
+        return self.embedding_similarity_metric(
+            source[compare_key], target[compare_key]
+        )
+
+
+class MinimalViewMixin:
+    minimal_view_mode = False
+
+    def generate_minimal_view_button(self, callbacks=[]):
+        button = Button(description="Toggle Minimal View")
+        button.layout.width = "200px"
+        button.on_click(self.toggle_minimal_view_mode)
+
+        for cb in callbacks:
+            button.on_click(cb)
+
+        return button
+
+    def toggle_minimal_view_mode(self, *args):
+        self.minimal_view_mode = not self.minimal_view_mode
+        self.update_grid()
 
 
 class ImageGridMixin:
@@ -220,6 +234,29 @@ class ImageGridMixin:
         # `_upper` exists to increase the upper index when looping, in case
         # we are hiding completed items
         _upper = end
+
+        df = None
+        if hasattr(self, "positive_class_map_filter"):
+            df = self.filter_dataset(self.class_map_subset_filter)
+
+        # Unload just the img bytes of items we aren't currently viewing
+        # if hasattr(self, "active_datapoints"):
+        #     # If this item is already in the grid and must remain there,
+        #     # then we don't unload it
+        #     if not self.class_map_positive_buttons.value == ():
+        #         dont_unload = np.intersect1d(
+        #             ([item.source for item in self.active_datapoints]),
+        #             df[self.filename_col].values,
+        #         )
+        #     else:
+        #         dont_unload = []
+
+        #     for item in self.active_datapoints:
+        #         if item.source in dont_unload:
+        #             pass
+        #         else:
+        #             item.unload()
+
         if hasattr(self, "positive_class_map_filter"):
             # This is what will almost always get executed
             df = self.filter_dataset(self.class_map_subset_filter)
@@ -228,9 +265,9 @@ class ImageGridMixin:
                     item = self.datapoints[fname]
                     if hasattr(self, "hide_completed") or hasattr(self, "hide_review"):
                         if (
-                            (item["item"].is_completed and self.hide_completed)
-                            or (item["item"].is_under_review and self.hide_review)
-                            or (item["item"].is_deleted and self.hide_deleted)
+                            (item.is_completed and self.hide_completed)
+                            or (item.is_under_review and self.hide_review)
+                            or (item.is_deleted and self.hide_deleted)
                         ):
                             _upper += 1
                             continue
@@ -239,12 +276,24 @@ class ImageGridMixin:
                     else:
                         items.append(item)
 
+                    if not item.is_loaded:
+                        item.load()
+                        item.update_view()
+                    elif item.is_loaded and item.needs_refresh:
+                        item.load_img_bytes()
+
+                    if hasattr(self, "minimal_view_mode"):
+                        if self.minimal_view_mode:
+                            item.update_view_minimal()
+                        else:
+                            item.update_view()
+
         else:
             for i, (key, item) in enumerate(self.datapoints.items()):
                 if i >= start and i < end:
                     items.append(item)
 
-        children = [i["view"] for i in items]
+        children = [i.view for i in items]
         self.active_datapoints = items
         self.grid.children = children
 
@@ -269,6 +318,9 @@ class ImageGridMixin:
             ).sum()
             if max_value > num_filtered_items:
                 max_value = num_filtered_items
+            if min_value > num_filtered_items:
+                # HACK ? reset min value...
+                min_value = 0
         max_limit = min(len(self), num_filtered_items)
 
         self.grid_range_slider.value = (min_value, max_value)
@@ -324,7 +376,7 @@ class InfoMixin:
         if isinstance(additional_info, str):
             additional_info = [additional_info]
 
-        num_images_in_grid = len(self.grid.children) if hasattr(self, "grid") else 0
+        num_images_in_grid = len(getattr(self, self.selection_source))
         if hasattr(self, "class_map_positive_buttons"):
             num_filtered_images = (
                 self.positive_class_map_filter & self.negative_class_map_filter
@@ -337,6 +389,7 @@ class InfoMixin:
             f"Total: {len(self)}",
             f"Deleted: {self.num_deleted}",
             f"Modified: {self.num_modified}",
+            f"Looked At: {self.num_loaded}/{len(self)}",
             f"Completed: {self.num_completed}/{len(self)}",
         ]
 
