@@ -3,8 +3,10 @@ from .review_mixins import *
 from .filtering_mixins import *
 from .dataset_mixins import *
 from .mark_mixins import *
+from .stats_mixins import *
 from datawidgets.data import *
 from datawidgets.interface import *
+from datawidgets.utils import *
 
 
 class ImageDataset(
@@ -14,6 +16,9 @@ class ImageDataset(
     SelectionMixin,
     InfoMixin,
 ):
+    active_datapoints = []
+    review_datapoints = []
+
     def __init__(
         self, df, batch_size: int = 50, width: int = 100, filename_col: str = "filename"
     ):
@@ -24,9 +29,18 @@ class ImageDataset(
         self.width = width
         # self.num_deleted = 0
 
+        self.init_css()
         self.setup()
         self.setup_view()
 
+    def init_css(self):
+        CSS = f"""
+        <style>
+        {Path("../datawidgets/ui/style.css").read_text()}
+        </style>
+        """
+        display(HTML(CSS))
+
     def setup_items(self):
         items = {}
         for idx, fname in tqdm(
@@ -35,11 +49,8 @@ class ImageDataset(
             desc="Setup Data Items",
         ):
             item = ImageDataPoint(fname)
-            items[idx] = {
-                "item": item,
-                "view": item.view,
-            }
-        self.datapoints = items
+            items[idx] = item
+        self.datapoints = pd.Series(items)
 
     @staticmethod
     def _check_filter(filter_: Union[Sequence[str], np.ndarray, pd.Series]):
@@ -68,16 +79,13 @@ class ImageDataset(
         for i in filter_:
             # del self.datapoints[i]
             items[i] = self.datapoints[i]
-        self.datapoints = items
+        self.datapoints = pd.Series(items)
         self.df = self.df.loc[filter_]
 
     def refresh(self):
         self.update_grid()
         self.update_info()
 
-    # def __repr__(self):
-    #     self._repr_html_()
-
     def setup(self):
         self.setup_items()
         self.setup_logging()
@@ -95,17 +103,15 @@ class ImageDataset(
 
     @property
     def num_modified(self):
-        return sum([item["item"].is_modified for item in self.datapoints.values()])
+        return sum([item.is_modified for item in self.datapoints])
 
     @property
     def num_deleted(self):
-        return sum([item["item"].is_deleted for item in self.datapoints.values()])
+        return sum([item.is_deleted for item in self.datapoints])
 
     @property
     def num_selected(self):
-        num_selected = sum(
-            [item["item"].is_selected for item in self.datapoints.values()]
-        )
+        num_selected = sum([item.is_selected for item in self.datapoints])
         if hasattr(self, "similarity_button"):
             if num_selected > 1:
                 self.similarity_button.disabled = True
@@ -115,13 +121,25 @@ class ImageDataset(
 
     @property
     def selected_items(self):
-        return [
-            item["item"]
-            for item in self.datapoints.values()
-            if item["item"].is_selected
-        ]
+        return [item for item in self.datapoints if item.is_selected]
         # return [item for item in self.active_datapoints if item.is_selected]
 
+    @property
+    def num_loaded(self):
+        return sum([item.is_loaded for item in self.datapoints])
+
+    @property
+    def num_needs_refresh(self):
+        return sum([item.needs_refresh for item in self.datapoints])
+
+    @property
+    def loaded_items(self):
+        return [item for item in self.datapoints if item.is_loaded]
+
+    @property
+    def fully_loaded_items(self):
+        return [item for item in self.loaded_items if not item.needs_refresh]
+
 
 class ImageClassificationDataset(
     ImageDataset,
@@ -129,6 +147,8 @@ class ImageClassificationDataset(
     ClassMapFilterMixin,
     ReviewMixin,
     DownloadModifiedMixin,
+    LabelsStatsMixin,
+    MinimalViewMixin,
     MarkSelectedAsCompletedMixin,
     MarkSelectedAsReviewMixin,
     MarkSelectedAsDeletedMixin,
@@ -140,46 +160,25 @@ class ImageClassificationDataset(
         batch_size: int = 50,
         width: int = 25,
         class_map: Optional[ClassMap] = None,
+        additional_classes: Optional[List[str]] = [],
         label_col: str = "label",
         is_multilabel: bool = False,
     ):
+        # `additional_classes` are great for when you're auto configuring the
+        # class map but want to add a few more options
+        ""
+
         self.df = df
         self.label_col = label_col
+        self.filename_col = filename_col
         self.is_multilabel = is_multilabel
 
-        # Derive class map from `label_col` if not passed in
         if class_map is None:
-            # TODO: Multi-label class map
-            if not self.is_multilabel:
-                self.class_map = ClassMap(
-                    classes=list(self.df[self.label_col].unique()),
-                    background=None,
-                )
-            else:
-
-                def convert_labels_to_list(label: Union[str, List[str]]):
-                    if isinstance(label, str):
-                        return [label]
-                    elif isinstance(label, list):
-                        return label
-                    elif isinstance(label, np.ndarray):
-                        if isinstance(label[0], str):
-                            return list(label)
-                    raise TypeError(
-                        f"Expected string or list of labels, got {type(label)}"
-                    )
-
-                self.df[self.label_col] = self.df[self.label_col].apply(
-                    convert_labels_to_list
-                )
-                all_labels = self.df[self.label_col].values
-                self.class_map = ClassMap(
-                    classes=uniqueify(flatten(all_labels)),
-                    background=None,
-                )
+            self.class_map = self.parse_class_map_from_df(
+                additional_classes=additional_classes
+            )
         else:
             self.class_map = class_map
-
         self.classes = self.class_map._id2class
 
         # HACK: blah..
@@ -189,14 +188,43 @@ class ImageClassificationDataset(
         self.width = width
         self.width_slider.value = self.width
 
+    def parse_class_map_from_df(self, additional_classes: Optional[List[str]]):
+        """
+        Derives the ClassMap from all the unique values in `self.label_col`
+        """
+        self.df[self.label_col] = self.df[self.label_col].apply(convert_labels_to_list)
+        if not self.is_multilabel:
+
+            def truly_single_label(row: pd.Series):
+                label = row[self.label_col]
+                fname = row[self.filename_col]
+                if not len(label) == 1:
+                    raise ValueError(
+                        f"Expected only one label per image in single label mode, but '{fname}' "
+                        f"has {len(label)} labels: {label}"
+                    )
+
+            self.df.apply(truly_single_label, axis=1)
+
+        all_labels = self.df[self.label_col].values
+        return ClassMap(
+            classes=uniqueify(flatten(all_labels)) + additional_classes,
+            background=None,
+        )
+
     def get_results(self):
         # TODO: Use a named tuple or dataclass for rows
         rows = []
 
         # Iterate through all datapoints and find the completed / modified items
-        for fname, datapoint in self.datapoints.items():
+        for fname, item in self.datapoints.items():
+
+            # TODO: Should we load and show all datapoints?
+            # or skip the item altogether?
+            if not item.is_loaded:
+                continue
+                # item.load()
 
-            item = datapoint["item"]
             filename = fname
             labels = item.labels
             note = item.note.value
@@ -230,7 +258,7 @@ class ImageClassificationDataset(
         # Merge with main dataframe while replacing the `self.label_col`
         modified_df = modified_df.merge(
             self.df.drop(columns=[self.label_col]).reset_index(drop=True),
-            on="filename",
+            on=self.filename_col,
         )
         modified_df.index = modified_df[self.filename_col]
         return modified_df
@@ -253,10 +281,11 @@ class ImageClassificationDataset(
 
         self.update_info()
         self.update_batch_labelling_descriptions()
+        self.setup_labels_stats()
 
     @property
     def num_completed(self):
-        return sum([i["item"].is_completed for i in self.datapoints.values()])
+        return sum([item.is_completed for item in self.datapoints])
 
     def setup_items(self):
         items = {}
@@ -270,11 +299,8 @@ class ImageClassificationDataset(
                 is_multilabel=self.is_multilabel,
                 parent_dataset=self,
             )
-            items[idx] = {
-                "item": item,
-                "view": item.view,
-            }
-        self.datapoints = items
+            items[idx] = item
+        self.datapoints = pd.Series(items)
 
     def setup_view(self, global_callbacks=[]):
         callbacks = [
@@ -296,10 +322,13 @@ class ImageClassificationDataset(
         unselect_button = self.generate_unselect_all_button(callbacks)
         select_button = self.generate_select_all_button(callbacks)
         invert_selection_button = self.generate_invert_selection_button(callbacks)
+        self.minimal_view_button = self.generate_minimal_view_button()
 
-        refresh_review_button = self.generate_review_refresh_button()
+        refresh_review_modified_button = self.generate_review_modified_button()
         refresh_view_selected_button = self.generate_selected_refresh_button()
         refresh_review_completed_button = self.generate_review_completed_button()
+        refresh_review_review_button = self.generate_review_review_button()
+        refresh_review_deleted_button = self.generate_review_deleted_button()
         refresh_export_button = self.generate_export_refresh_button()
 
         self.class_map_filter_button.on_click(self.update_grid_range_slider)
@@ -322,15 +351,7 @@ class ImageClassificationDataset(
 
         self.set_grid_range_slider(view_range_slider)
 
-        status_controls = HBox(
-            # [
-            #     self.mark_deleted_button,
-            #     self.mark_completed_button,
-            #     self.mark_review_button,
-            #     self.mark_deleted_toggle,
-            #     self.mark_completed_toggle,
-            #     self.mark_review_toggle,
-            # ]
+        mark_controls = HBox(
             [
                 self.mark_deleted_controls,
                 self.mark_completed_contols,
@@ -340,14 +361,13 @@ class ImageClassificationDataset(
         selection_controls = HBox(
             [select_button, invert_selection_button, unselect_button]
         )
-        image_view_controls = HBox(
-            [
-                self.width_slider,
-                decrement_range,
-                self.grid_range_slider,
-                increment_range,
-            ]
-        )
+        item_display_controls = [
+            decrement_range,
+            self.grid_range_slider,
+            increment_range,
+        ]
+        image_view_controls = HBox([self.width_slider, *item_display_controls])
+        grid_view_controls = HBox([self.minimal_view_button])
         sorting_controls = HBox([self.similarity_button])
         batch_labelling_controls = HBox(
             [
@@ -357,15 +377,18 @@ class ImageClassificationDataset(
         )
         review_controls = HBox(
             [
-                refresh_review_button,
+                refresh_review_modified_button,
                 refresh_view_selected_button,
                 refresh_review_completed_button,
+                refresh_review_review_button,
+                refresh_review_deleted_button,
             ]
         )
 
-        status_controls.layout = CSS_LAYOUTS.flex_layout
+        mark_controls.layout = CSS_LAYOUTS.flex_layout
         selection_controls.layout = CSS_LAYOUTS.flex_layout
         image_view_controls.layout = CSS_LAYOUTS.flex_layout
+        grid_view_controls.layout = CSS_LAYOUTS.flex_layout
         sorting_controls.layout = CSS_LAYOUTS.flex_layout
         batch_labelling_controls.layout = CSS_LAYOUTS.flex_padded
         review_controls.layout = CSS_LAYOUTS.flex_layout
@@ -373,15 +396,52 @@ class ImageClassificationDataset(
         REVIEW_TAB = widgets.VBox(
             [
                 review_controls,
+                CSS_SPACERS.vspace(10),
+                HBox(
+                    [
+                        self.mark_deleted_button,
+                        self.mark_completed_button,
+                        self.mark_review_button,
+                    ],
+                    layout=CSS_LAYOUTS.flex_layout,
+                ),
+                CSS_SPACERS.vspace(5),
+                selection_controls,
+                CSS_SPACERS.vspace(5),
                 self.info,
                 HBox([self.width_slider], layout=CSS_LAYOUTS.flex_layout),
-                HBox([self.mark_completed_button], layout=CSS_LAYOUTS.flex_layout),
+                CSS_SPACERS.vspace(5),
                 batch_labelling_controls,
                 self.review_grid,
             ]
         )
 
-        refresh_export_button.click()
+        (
+            self.refresh_total_labels_stats,
+            self.refresh_filtered_labels_stats,
+            self.refresh_selected_labels_stats,
+            self.refresh_active_datapoints_stats,
+            self.refresh_review_datapoints_stats,
+        ) = self.generate_refresh_stats_button(callbacks=[])
+        self.stats_controls = HBox(
+            [
+                self.refresh_total_labels_stats,
+                self.refresh_filtered_labels_stats,
+                self.refresh_selected_labels_stats,
+                self.refresh_active_datapoints_stats,
+                self.refresh_review_datapoints_stats,
+            ]
+        )
+        self.stats_controls.layout = CSS_LAYOUTS.flex_layout
+
+        STATS_TAB = VBox(
+            [
+                HBox([self.stats_controls]),
+                self.stats_output_area,
+            ]
+        )
+
+        # refresh_export_button.click()
         refresh_export_button_centered = HBox([refresh_export_button])
         refresh_export_button_centered.layout = CSS_LAYOUTS.flex_layout
         EXPORT_TAB = VBox(
@@ -391,50 +451,90 @@ class ImageClassificationDataset(
             ]
         )
 
+        # fmt: off
         MAIN_CONTROLS = widgets.VBox(
             [
                 self.class_map_filtering_controls,
-                image_view_controls,
-                status_controls,
-                selection_controls,
-                sorting_controls,
+                image_view_controls, CSS_SPACERS.vspace(10),
+                mark_controls,       CSS_SPACERS.vspace(10),
+                selection_controls,  CSS_SPACERS.vspace(2),
+                sorting_controls,    CSS_SPACERS.vspace(2),
+                grid_view_controls,  CSS_SPACERS.vspace(5),
                 batch_labelling_controls,
             ]
         )
-
-        MAIN_OUTPUT = widgets.Tab(
+        # fmt: on
+        MAIN_TAB = widgets.VBox(
             [
-                widgets.VBox([MAIN_CONTROLS, self.info, self.grid]),
+                MAIN_CONTROLS,
+                self.info,
+                self.grid,
+                HBox(item_display_controls, layout=CSS_LAYOUTS.flex_layout),
+            ]
+        )
+
+        self.view = widgets.Tab(
+            [
+                MAIN_TAB,
                 REVIEW_TAB,
+                STATS_TAB,
                 EXPORT_TAB,
             ]
         )
 
-        MAIN_OUTPUT.set_title(0, "Main Labelling Main Labelling")
-        MAIN_OUTPUT.set_title(1, "Review")
-        MAIN_OUTPUT.set_title(2, "Export")
-        self.view = MAIN_OUTPUT
+        self.view.set_title(0, "Main Labelling")
+        self.view.set_title(1, "Review")
+        self.view.set_title(2, "Stats")
+        self.view.set_title(3, "Export")
+
+        # Define behavior when tabs are changed
+        def swap_selection_source(*args):
+
+            if self.view.selected_index == 0:
+                self.selection_source = "active_datapoints"
+            if self.view.selected_index == 1:
+                self.selection_source = "review_datapoints"
+
+            self.update_info()
+
+        self.view.observe(swap_selection_source, "selected_index")
 
 
-class CinemaNetDataset(ImageClassificationDataset, CinemaNetSimilarityMixin):
+from scipy import spatial
+
+
+class FeatureEmbeddedDataset(ImageClassificationDataset, EmbeddingSimilarityMixin):
     def __init__(
         self,
         df,
         batch_size: int = 50,
         width: int = 25,
         class_map: Optional[ClassMap] = None,
+        additional_classes: Optional[List[str]] = [],
+        embedding_colname: Union[str, List[str]] = "extracted_features",
+        embedding_similarity_metric=spatial.distance.cosine,
         label_col: str = "label",
         is_multilabel: bool = False,
     ):
+        if isinstance(embedding_colname, str):
+            embedding_colname = [embedding_colname]
+        self.embedding_colname = embedding_colname
+        self.embedding_similarity_metric = embedding_similarity_metric
+
         super().__init__(
             df=df,
             batch_size=batch_size,
             width=width,
             label_col=label_col,
             class_map=class_map,
+            additional_classes=additional_classes,
             is_multilabel=is_multilabel,
         )
 
     def setup(self):
         super().setup()
         self.setup_similarity()
+
+
+# tmp backward compat
+CinemaNetDataset = FeatureEmbeddedDataset
