@@ -19,7 +19,7 @@ class AbstractInterface(ABC):
 
     def __init__(self, source: Union[str, Path, Any], width: int = 100):
         # Store filepath, create image, setup logs
-        self.source = source
+        self.source = str(source)
         self.width = width
 
         self.setup()
@@ -27,6 +27,9 @@ class AbstractInterface(ABC):
         self.setup_view()
         self.update_view()
 
+    def load(self):
+        pass
+
     @abstractmethod
     def setup(self):
         pass
