@@ -51,6 +51,14 @@ class CSS_LAYOUTS:
     right_aligned = Layout(align_items="flex-end")
 
 
+class CSS_SPACERS:
+    def vspace(height: int = 10):
+        return HBox([], layout=Layout(height=f"{height}px"))
+
+    def hspace(width: int = 10):
+        return HBox([], layout=Layout(width=f"{width}px"))
+
+
 class CSS_NAMES:
     IMG_BOX = "image-box"
     IMG_COMPLETED = "image-box-completed"
@@ -91,6 +99,7 @@ class CSS_NAMES:
         self.IMG_BOX_HOVER = f"{self.IMG_BOX}-on-hover"
         self.IMG_BOX_SELECTED = f"{self.IMG_BOX}-selected"
         self.IMG_BOX_CONTAINER = f"{self.IMG_BOX}-container"
+        self.IMG_BOX_CONTAINER_MINIMAL = f"{self.IMG_BOX_CONTAINER}-minimal"
         self.IMG_BOX_CONTAINER_SELECTED = f"{self.IMG_BOX_CONTAINER}-selected"
         self.FILE_UPLOAD_REFRESH_BUTTON = f"{self.FILE_UPLOAD_BUTTON}-refresh"
         self.FILE_UPLOAD_CONTAINER = f"{self.FILE_UPLOAD_BUTTON}-container"
